"""
Abstract setup module
"""

import abc
import argparse

from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.structure_generator import StructureGenerator


class Setup:
    """
    Abstract setup class.
    """

    def __init__(self, args: argparse.Namespace, cwd: str) -> None:
        """
        Initializes setup class with proper parameters.
        """
        self.args = args
        self.cwd = cwd

    @abc.abstractmethod
    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """
        raise NotImplementedError("Unimplemented")

    @abc.abstractmethod
    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """
        raise NotImplementedError("Unimplemented")

    @abc.abstractmethod
    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """
        raise NotImplementedError("Unimplemented")



"""
DVC generator module.
"""
import argparse
import subprocess  # nosec

from rpgt import templates
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.generator import Generator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.utils.file_reader import get_file_contents


class DVCGenerator(Generator):
    """
    DVC generator class.
    """

    def __init__(
        self,
        args: argparse.Namespace,
        env_gen: EnvironmentGenerator,
        struct_gen: StructureGenerator,
    ):
        """
        Abstract initialize function definition.
        """
        super().__init__(args)
        self.struct_gen = struct_gen
        env_gen.add_requirement("dvc", "~=2.3.0", dev_only=True)

        field = self.args.field
        if field == "RL":
            filename = "rl.yaml"
        elif field == "DL":
            filename = "dl.yaml"
        elif field == "ML":
            filename = "ml.yaml"
        else:
            raise Exception(f"'{field}' is not supported as a field")

        self.struct_gen.add_file(
            "dvc.yaml",
            get_file_contents(templates.dvc, filename),
        )

    def generate(self, cwd: str) -> None:
        """
        Generates the DVC configuration file.
        """
        with subprocess.Popen(  # nosec
            ["dvc", "init"], stdout=subprocess.PIPE
        ) as process:
            process.wait()



"""
Abstract generator module.
"""

import abc
import argparse


class Generator:
    """
    Abstract generator class.
    """

    def __init__(self, args: argparse.Namespace):
        """
        Abstract initialize function definition.
        """
        self.args = args

    @abc.abstractmethod
    def generate(self, cwd: str) -> None:
        """
        Abstract generate function definition.
        """
        raise NotImplementedError("Unimplemented")



"""
Abstract env generator module.
"""

import argparse

from rpgt.src.setup.environment.environment_setup import EnvironmentSetup
from rpgt.src.setup.environment.pipenv_setup import PipEnvSetup


def get_env(name: str, args: argparse.Namespace, cwd: str) -> EnvironmentSetup:
    """
    Get env setup.
    :return: the env setup.
    """
    # Environment
    if name == "pipenv":
        env_setup: EnvironmentSetup = PipEnvSetup(args, cwd)
    else:
        raise Exception(f"'{name}' is not supported as an environment")

    return env_setup



"""
 Pytest setup module.
 """

from rpgt import templates
from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.setup import Setup
from rpgt.src.utils.file_reader import get_file_contents


class PytestSetup(Setup):
    """
    Pytest setup class.
    """

    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """
        env_gen.add_requirement("pytest", "~=6.2.4", dev_only=True)
        env_gen.add_requirement("pytest-html", "~=3.1.1", dev_only=True)

    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """
        struct_gen.add_folder("tests", True)
        struct_gen.add_folder("tests/unit_tests", True)
        struct_gen.add_folder(f"tests/unit_tests/{self.args.name}", True)
        struct_gen.add_folder(f"tests/unit_tests/{self.args.name}/utils", True)

        # Example test
        struct_gen.add_unit_test_file(
            "utils/test_math_utils.py",
            get_file_contents(templates.tests.unit, "test_math_utils.txt"),
        )

    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """
        cicd_gen.add_job(
            "unit-test",
            {
                "stage": "test",
                "script": [
                    "pipenv run pytest --html=unit-report.html --junitxml=report.xml tests/unit_tests",
                ],
                "artifacts": [
                    {
                        "paths": ["unit-report.html", "report.xml"],
                        "expire_in": "1 month",
                        "reports": {"junit": ["report.xml"]},
                    }
                ],
            },
        )



"""
Git hooks generator module.
"""

import os
import stat
import subprocess  # nosec

from rpgt.src.generators.generator import Generator


class GitHooksGenerator(Generator):
    """
    Git hooks generator class.
    """

    def generate(self, cwd: str) -> None:
        """
        Generates the git hooks files.
        """
        # generate a .git folder
        with subprocess.Popen(  # nosec
            ["git", "init"], stdout=subprocess.PIPE
        ) as process:
            process.wait()

        # symlink the githooks folder to the hookspath
        with subprocess.Popen(  # nosec
            ["git", "config", "--local", "core.hooksPath", ".githooks/"],
            stdout=subprocess.PIPE,
        ) as process:
            process.wait()

        # pre-commit file
        file_path = f"{cwd}/.githooks/pre-commit"
        with open(file_path, "w") as file:
            file.write(self.generate_pre_commit())

        # make the file executable
        stat_call = os.stat(file_path)
        os.chmod(file_path, stat_call.st_mode | stat.S_IEXEC)

    def generate_pre_commit(self) -> str:
        """
        Generates pre commit message.
        :return: a stringual representation of the pre commit messag.
        """
        text = ["#!/bin/sh"]

        if self.args.lint:
            # Auto fix import sorting issues
            text.append("pipenv run isort .")
            text.append("pipenv run black .")

        return "\n".join(text)



"""
Command line interface
"""
import argparse
import os
import sys


def get_parser() -> argparse.ArgumentParser:
    """
    Creates a parser which parses the arguments and returns the parser.
    :return: the parser
    """
    parser = argparse.ArgumentParser(
        description="Generator to automatically set up your new project!"
    )

    # name
    parser.add_argument(
        "-n",
        "--name",
        metavar="name",
        dest="name",
        type=str,
        required=True,
        help="Name of the project.",
    )

    # target directory
    parser.add_argument(
        "-d",
        "--directory",
        metavar="target_directory",
        dest="target_directory",
        type=str,
        required=True,
        help="Directory where your project will generated.",
    )

    # field
    fields = ["ML", "RL", "DL"]
    parser.add_argument(
        "-f",
        "--field",
        metavar="field",
        dest="field",
        type=str,
        default="ML",
        choices=fields,
        help=f"Research field of the project. Allowed values are {', '.join(fields)}.",
    )

    # license
    licenses = ["agpl30", "apache20", "gpl30", "lgpl30", "mit", "mpl20", "unlicense"]
    parser.add_argument(
        "-l",
        "--license",
        metavar="license",
        dest="license",
        type=str,
        default="mit",
        choices=licenses,
        help=f"License of the project. Allowed values are {', '.join(licenses)}.",
    )

    # version control
    vcs = ["gitlab", "github"]
    parser.add_argument(
        "-g",
        "--version-control",
        metavar="",
        dest="version_control",
        type=str,
        default="gitlab",
        choices=vcs,
        help=f"Version control of the project. Allowed values are {', '.join(vcs)}.",
    )

    # environment
    envs = ["pipenv"]
    parser.add_argument(
        "-e",
        "--environment",
        metavar="",
        dest="environment",
        type=str,
        default="pipenv",
        choices=envs,
        help=f"Environment of the project. Allowed values are {', '.join(envs)}.",
    )

    # package
    parser.add_argument(
        "-p",
        "--package",
        metavar="value",
        dest="package",
        type=bool,
        default=True,
        help="Whether the tool is a package.",
    )

    # linting
    parser.add_argument(
        "-i",
        "--lint",
        metavar="value",
        dest="lint",
        type=bool,
        default=True,
        help="Whether the tool should be linted.",
    )

    # testing
    parser.add_argument(
        "-t",
        "--test",
        metavar="value",
        dest="test",
        type=bool,
        default=True,
        help="Whether the tool should be tested.",
    )

    return parser


def start_cli() -> argparse.Namespace:
    """
    Starts the cli program.
    :return:
    """
    parser = get_parser()
    args = parser.parse_args()

    args.name = args.name.replace("-", "_")

    target_directory = args.target_directory.replace("-", "_")
    dir_path = f"{target_directory}"
    if not os.path.exists(dir_path):
        os.mkdir(dir_path)
    os.chdir(dir_path)

    cwd = os.getcwd()

    # Check if current working directory is empty
    if len(os.listdir(cwd)) != 0:
        answer = input(
            f"The current working directory is not empty! Directory: {target_directory}\n"
            "Everything inside will be removed! Are you sure you want to continue? (y/n)\n"
        )
        if answer != "y":
            sys.exit(0)

    return args



"""
MLLint setup module.
"""

from rpgt import templates
from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.setup import Setup
from rpgt.src.utils.file_reader import get_file_contents


class MLLintSetup(Setup):
    """
    MLLint setup class.
    """

    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """
        env_gen.add_requirement("mllint", "~=0.9.0", dev_only=True)
        env_gen.add_requirement("isort", "~=5.8.0", dev_only=True)
        env_gen.add_requirement("pylint", "~=2.8.2", dev_only=True)
        env_gen.add_requirement("mypy", "~=0.812", dev_only=True)
        env_gen.add_requirement("black", "==21.5b2", dev_only=True)
        env_gen.add_requirement("bandit", "~=1.7.0", dev_only=True)

    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """
        struct_gen.add_file(
            ".bandit", get_file_contents(templates.configs, "bandit.txt")
        )
        struct_gen.add_file(
            ".mypy.ini", get_file_contents(templates.configs, "mypy.txt")
        )
        struct_gen.add_file(
            "pyproject.toml", get_file_contents(templates.configs, "pyproject.txt")
        )
        struct_gen.add_file(
            ".pylintrc", get_file_contents(templates.configs, "pylintrc.txt")
        )

    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """
        cicd_gen.add_job(
            "mllint",
            {
                "stage": "analysis",
                "script": ["pipenv run mllint -o mllint-report.md"],
                "artifacts": [{"paths": ["mllint-report.md"], "expire_in": "1 month"}],
            },
        )



"""
Gitlab CI generator module.
"""

from typing import Any

from rpgt.src.generators.cicd.cicd_generator import CICDGenerator


class GitlabCICDGenerator(CICDGenerator):
    """
    Gitlab CI generator class.
    """

    def add_job(self, job: str, data: Any) -> None:
        """
        Add job to CI
        """
        self.jobs[job] = data

        if data["stage"] not in self.stages:
            self.stages.append(data["stage"])

    def generate(self, cwd: str) -> None:
        """
        Generate CI
        """
        texts = ["image: python:3.8\n"]

        texts.append("variables:")
        texts.append("  WORKON_HOME: .pipenv/venvs")
        texts.append("  PIP_CACHE_DIR: .pipenv/pipcache\n")

        texts.append("cache:")
        texts.append("  paths:")
        texts.append("    - .cache/pip")
        texts.append("    - venv/")
        texts.append("    - .pipenv")
        texts.append("    - .venv/\n")

        texts.append("before_script:")
        texts.append("  - pip install pipenv\n")

        texts.append("stages:")
        for stage in self.stages:
            texts.append(f"  - {stage}")

        texts.append("")

        for job, data in self.jobs.items():
            texts.append(self.generate_job(job, data))

        final_text = "\n".join(texts)

        with open(f"{cwd}/.gitlab-ci.yml", "w") as file:
            file.write(final_text)

    def generate_job(self, job: str, data: Any) -> str:
        """
        Generate job
        """
        texts = [f"{job}:"]

        texts.append(f'  stage: {data["stage"]}')
        texts.append("  script:")

        for script_line in data["script"]:
            texts.append(f"    - {script_line}")

        if "artifacts" in data:
            texts.append("  artifacts:")
            for artifact in data["artifacts"]:
                texts.append("    paths:")
                for path in artifact["paths"]:
                    texts.append(f"      - {path}")
                texts.append(f'    expire_in: {artifact["expire_in"]}')

                if "reports" in artifact:
                    texts.append("    reports:")
                    texts.append("      junit:")
                    for report in artifact["reports"]["junit"]:
                        texts.append(f"        - {report}")

        if "only" in data:
            texts.append("  only:")
            for branch in data["only"]:
                texts.append(f"    - {branch}")

        texts.append("\n")
        return "\n".join(texts)



"""
Abstract CICD generator module.
"""

import abc
import argparse
from typing import Any, Dict, List

from rpgt.src.generators.generator import Generator


class CICDGenerator(Generator):
    """
    Abstract CICD generator class.
    """

    def __init__(self, args: argparse.Namespace):
        """
        Initialize CICD generator with proper parameters.
        """

        super().__init__(args)
        self.stages: List[str] = []
        self.jobs: Dict[str, Any] = dict()

    @abc.abstractmethod
    def add_job(self, job: str, data: Any) -> None:
        """
        Add job to CI
        """
        raise NotImplementedError("Unimplemented")

    @abc.abstractmethod
    def generate_job(self, job: str, data: Any) -> str:
        """
        Generate job
        """
        raise NotImplementedError("Unimplemented")



"""
Example math utils file
"""


def multiply(first: int, second: int) -> int:
    """
    Multiplication function
    :param first: first integer to multiply
    :param second: second integer to multiply
    :return: the product of the two integers
    """
    return first * second



"""
VCP generator module.
"""

import argparse

from rpgt.src.setup.vcp.github_setup import GithubSetup
from rpgt.src.setup.vcp.gitlab_setup import GitlabSetup
from rpgt.src.setup.vcp.vcp_setup import VCPSetup


def get_vcp(name: str, args: argparse.Namespace, cwd: str) -> VCPSetup:
    """
    Get VCP setup.
    :return: the VCP setup.
    """
    vcp_setup: VCPSetup
    # Version Control
    if name == "gitlab":
        vcp_setup = GitlabSetup(args, cwd)
    elif name == "github":
        print("WARNING: github is not fully supported yet!")
        vcp_setup = GithubSetup(args, cwd)
    else:
        raise Exception(f"'{name}' is not supported as a version control provider")

    return vcp_setup



"""
Field generator module.
"""

import argparse

from rpgt.src.setup.field.dl_setup import DLSetup
from rpgt.src.setup.field.field_setup import FieldSetup
from rpgt.src.setup.field.ml_setup import MLSetup
from rpgt.src.setup.field.rl_setup import RLSetup


def get_field(name: str, args: argparse.Namespace, cwd: str) -> FieldSetup:
    """
    Get field setup.
    :return: the field setup.
    """
    field_setup: FieldSetup
    # Field
    if name == "RL":
        field_setup = RLSetup(args, cwd)
    elif name == "DL":
        field_setup = DLSetup(args, cwd)
    elif name == "ML":
        field_setup = MLSetup(args, cwd)
    else:
        raise Exception(f"'{name}' is not supported as a field")

    return field_setup



# no-redef: ignore
"""
File reader module.
"""

try:
    import importlib.resources as pkg_resources
except ImportError:
    # Try backported to PY<37 `importlib_resources`.
    import importlib_resources as pkg_resources


def get_file_contents(folder, filename: str) -> str:  # type: ignore
    """
    Reads the contents of a file and returns it.
    :param folder:
    :param filename:
    :return:
    """
    return pkg_resources.read_text(folder, filename)  # pylint: disable=broad-except



"""
License generator module.
"""

import argparse

from rpgt import templates
from rpgt.src.generators.generator import Generator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.utils.file_reader import get_file_contents


class LicenseGenerator(Generator):
    """
    License generator class.
    """

    def __init__(self, args: argparse.Namespace, struct_gen: StructureGenerator):
        """
        Initializes the license generator.
        """
        super().__init__(args)
        self.struct_gen = struct_gen

        self.license_templates = [
            "agpl30",
            "apache20",
            "gpl30",
            "mit",
            "mpl20",
            "unlicense",
        ]

    def generate(self, cwd: str) -> None:
        """
        Writes the LICENSE.md to file.
        """
        #   | AGPL30       | agpl30            |
        #   | APACHE20     | apache20          |
        #   | GPL30        | gpl30             |
        #   | LGPL30       | lgpl30            |
        #   | MIT          | mit               |
        #   | MPL20        | mpl20             |
        #   | UNLICENSE    | unlicense         |

        license_type = self.args.license.lower()
        if license_type not in self.license_templates:
            raise Exception(f"License {license_type} is not supported")

        self.struct_gen.add_file(
            "LICENSE.md", get_file_contents(templates.licenses, f"{license_type}.txt")
        )



"""
Changelog generator module.
"""
import argparse
import datetime

from rpgt.src.generators.generator import Generator
from rpgt.src.generators.structure_generator import StructureGenerator


class ChangelogGenerator(Generator):
    """
    Changelog generator class.
    """

    def __init__(self, args: argparse.Namespace, struct_gen: StructureGenerator):
        """
        Initialize Changelog generator with proper parameters.
        """

        super().__init__(args)
        self.struct_gen = struct_gen

    def generate(self, cwd: str) -> None:
        """
        Writes the CHANGELOG.md to file.
        """
        self.struct_gen.add_file("CHANGELOG.md", self.generate_changelog())

    @staticmethod
    def generate_changelog() -> str:
        """
        Generates the CHANGELOG.md file.
        :return: a stringual representation of the changelog file.
        """
        return f"""# Changelog

All notable changes to this project will be documented in this file.

The format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),
and this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).

## 1.0.0 - {datetime.datetime.now().strftime('%Y-%m-%d')}
### Added
- Initial project setup.

### Changed
- None

### Fixed
- None
"""



"""
Main file
"""
import argparse
import os
from typing import List

from rpgt import templates
from rpgt.src.cli.cli import start_cli
from rpgt.src.factories.env_factory import get_env
from rpgt.src.factories.field_factory import get_field
from rpgt.src.factories.vcp_factory import get_vcp
from rpgt.src.generators.changelog_generator import ChangelogGenerator
from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.contributing_generator import ContributingGenerator
from rpgt.src.generators.dockerfile_generator import DockerfileGenerator
from rpgt.src.generators.dvc_generator import DVCGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.generator import Generator
from rpgt.src.generators.githooks_generator import GitHooksGenerator
from rpgt.src.generators.licence_generator import LicenseGenerator
from rpgt.src.generators.readme_generator import ReadmeGenerator
from rpgt.src.generators.setup_py_generator import SetupGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.mllint_setup import MLLintSetup
from rpgt.src.setup.package_setup import PackageSetup
from rpgt.src.setup.pytest_setup import PytestSetup
from rpgt.src.setup.setup import Setup
from rpgt.src.utils.file_reader import get_file_contents
from rpgt.src.utils.file_system import clear_cwd


def main() -> None:
    """
    Main function
    """
    args: argparse.Namespace = start_cli()
    cwd = os.getcwd()

    generators: List[Generator] = []
    setups: List[Setup] = []

    struct_gen = StructureGenerator(args)

    # Field
    setups.append(get_field(args.field, args, cwd))

    # VCP
    vcp_setup = get_vcp(args.version_control, args, cwd)
    setups.append(vcp_setup)
    cicd_gen: CICDGenerator = vcp_setup.get_standard_pipeline()

    # ENV
    env_setup = get_env(args.environment, args, cwd)
    setups.append(env_setup)
    env_gen: EnvironmentGenerator = env_setup.get_standard_env()

    # generators
    # these only configure
    generators.extend(
        [
            LicenseGenerator(args, struct_gen),
            ReadmeGenerator(args, env_gen),
            ContributingGenerator(args, struct_gen),
            ChangelogGenerator(args, struct_gen),
            DockerfileGenerator(args, struct_gen),
        ]
    )

    # these actually generate
    generators.extend([struct_gen, env_gen, cicd_gen])
    generators.append(GitHooksGenerator(args))
    generators.append(DVCGenerator(args, env_gen, struct_gen))

    add_base_files_and_folders(args, struct_gen)

    clear_cwd(cwd)

    # Lint
    if args.lint:
        mllint_setup = MLLintSetup(args, cwd)
        setups.append(mllint_setup)

    # Tests
    if args.test:
        pytest_setup = PytestSetup(args, cwd)
        setups.append(pytest_setup)

    # Package
    if args.package:
        package_setup = PackageSetup(args, cwd)
        setups.append(package_setup)
        setup_gen = SetupGenerator(args, env_gen)
        setup_gen.generate(cwd)
        generators.append(setup_gen)

    for setup in setups:
        setup.setup_dependencies(env_gen)
        setup.setup_files(struct_gen)
        setup.setup_pipeline(cicd_gen)

    for generator in generators:
        generator.generate(cwd)

    print(
        "All set! "
        "Please follow the instructions in the 'Next steps' section on:\n"
        "https://gitlab.com/tweet.dimitri/rpgt/-/wikis/Next%20steps"
    )


def add_base_files_and_folders(
    args: argparse.Namespace, struct_gen: StructureGenerator
) -> None:
    """
    Add the minimally required folders and files
    :param args:
    :param struct_gen:
    :return:
    """
    # Directories
    struct_gen.add_folder("src", False)
    struct_gen.add_folder(f"src/{args.name}", True)
    struct_gen.add_folder(f"src/{args.name}/utils", True)

    struct_gen.add_folder("data", False)
    struct_gen.add_folder("results", False)
    struct_gen.add_folder("results/plots", False)
    struct_gen.add_folder("results/models", False)
    struct_gen.add_folder(".githooks", False)

    struct_gen.add_folder("preprocessed_data", False)

    # Code
    struct_gen.add_src_file(
        "utils/math_utils.py", get_file_contents(templates.code, "math_utils.py")
    )

    # Static files
    struct_gen.add_file(
        ".gitignore", get_file_contents(templates.gitignore, "gitignore.txt")
    )


if __name__ == "__main__":
    main()



"""
Gitlab setup module.
"""

from rpgt import templates
from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.cicd.gitlab_ci_generator import GitlabCICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.vcp.vcp_setup import VCPSetup
from rpgt.src.utils.file_reader import get_file_contents


class GitlabSetup(VCPSetup):
    """
    Gitlab setup class.
    """

    def get_standard_pipeline(self) -> GitlabCICDGenerator:
        """
        Sets up project pipeline.
        """
        return GitlabCICDGenerator(self.args)

    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """
        # Directories
        struct_gen.add_folder(".gitlab", False)
        struct_gen.add_folder(".gitlab/issue_templates", False)
        struct_gen.add_folder(".gitlab/merge_request_templates", False)

        # version_control files
        struct_gen.add_file(
            ".gitlab/issue_templates/Bug.md",
            get_file_contents(templates.version_control.issue_templates, "Bug.md"),
        )
        struct_gen.add_file(
            ".gitlab/issue_templates/Feature.md",
            get_file_contents(templates.version_control.issue_templates, "Feature.md"),
        )
        struct_gen.add_file(
            ".gitlab/merge_request_templates/default.md",
            get_file_contents(
                templates.version_control.merge_request_templates, "default.md"
            ),
        )

    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """

    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """



"""
Github CI generator module.
"""

from typing import Any

from rpgt.src.generators.cicd.cicd_generator import CICDGenerator


class GithubCICDGenerator(CICDGenerator):
    """
    Gitlab CI generator class.
    """

    def add_job(self, job: str, data: Any) -> None:
        """
        Add job to CI
        """

    def generate(self, cwd: str) -> None:
        """
        Generate CI
        """

    def generate_job(self, job: str, data: Any) -> str:
        """
        Generate job
        """



"""
Abstract env generator module.
"""

import abc
import argparse
from typing import Any, Dict

from rpgt.src.generators.generator import Generator


class EnvironmentGenerator(Generator):
    """
    Abstract env generator class.
    """

    def __init__(self, args: argparse.Namespace):
        """
        Initialize env generator with proper parameters.
        """
        super().__init__(args)
        self.requirements: Dict[str, Any] = dict()

    @abc.abstractmethod
    def add_requirement(
        self, requirement: str, version: str, dev_only: bool = False
    ) -> None:
        """
        Add requirement to env.
        """
        raise NotImplementedError("Unimplemented")



"""
Abstract VCP setup module.
"""
import abc

from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.setup.setup import Setup


class VCPSetup(Setup):
    """
    Abstract VCP setup class.
    """

    @abc.abstractmethod
    def get_standard_pipeline(self) -> CICDGenerator:
        """
        Sets up project pipeline definition.
        """
        raise NotImplementedError("Unimplemented")



"""
Pipenv generator module.
"""

import argparse

from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator


class PipEnvGenerator(EnvironmentGenerator):
    """
    Pipenv generator class.
    """

    def __init__(self, args: argparse.Namespace):
        """
        Initialize pipenv generator with proper parameters.
        """
        super().__init__(args)
        self.add_requirement("pipenv", "~=2021.5.29", dev_only=True)

    def add_requirement(
        self, requirement: str, version: str, dev_only: bool = False
    ) -> None:
        """
        Add requirement to env.
        """
        self.requirements[requirement] = {"version": version, "dev_only": dev_only}

    def generate(self, cwd: str) -> None:
        """
        Generate the pipenv.
        """
        text = []

        text.append("[[source]]")
        text.append('url = "https://pypi.org/simple"')
        text.append("verify_ssl = true")
        text.append('name = "pypi"\n')

        text.append("[packages]")

        for requirement, data in self.requirements.items():
            if not data["dev_only"]:
                text.append(f'{requirement} = "{data["version"]}"')

        text.append("\n[dev-packages]")

        for requirement, data in self.requirements.items():
            if data["dev_only"]:
                text.append(f'{requirement} = "{data["version"]}"')

        text.append("\n[requires]")
        text.append('python_version = "3.8"')

        with open(f"{cwd}/Pipfile", "w") as file:
            file.write("\n".join(text))



"""
Templates module.
"""

import rpgt.templates.code
import rpgt.templates.code.pytorch
import rpgt.templates.code.sklearn
import rpgt.templates.code.stablebaselines
import rpgt.templates.configs
import rpgt.templates.docker
import rpgt.templates.dvc
import rpgt.templates.gitignore
import rpgt.templates.licenses
import rpgt.templates.tests
import rpgt.templates.tests.unit
import rpgt.templates.version_control.issue_templates
import rpgt.templates.version_control.merge_request_templates



"""
DL setup module.
"""

from rpgt import templates
from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.field.field_setup import FieldSetup
from rpgt.src.utils.file_reader import get_file_contents


class DLSetup(FieldSetup):
    """
    DL setup class.
    """

    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """
        env_gen.add_requirement("torch", "*", dev_only=False)
        env_gen.add_requirement("torchvision", "*", dev_only=False)

    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """
        struct_gen.add_src_file(
            "get_data.py",
            get_file_contents(templates.code.pytorch, "get_data.txt"),
        )
        struct_gen.add_src_file(
            "model.py",
            get_file_contents(templates.code.pytorch, "model.txt"),
        )
        struct_gen.add_src_file(
            "train.py",
            get_file_contents(templates.code.pytorch, "train.txt"),
        )
        struct_gen.add_src_file(
            "validate.py",
            get_file_contents(templates.code.pytorch, "validate.txt"),
        )

    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """



"""
RL setup module.
"""

from rpgt import templates
from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.field.field_setup import FieldSetup
from rpgt.src.utils.file_reader import get_file_contents


class RLSetup(FieldSetup):
    """
    RL setup class.
    """

    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """
        env_gen.add_requirement("stable-baselines3", "*", dev_only=False)
        env_gen.add_requirement("gym", "~=0.18.3", dev_only=False)

    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """
        struct_gen.add_src_file(
            "__main__.py",
            get_file_contents(templates.code.stablebaselines, "__main__.txt"),
        )
        struct_gen.add_src_file(
            "model_loader.py",
            get_file_contents(templates.code.stablebaselines, "model_loader.txt"),
        )
        struct_gen.add_src_file(
            "get_environment.py",
            get_file_contents(templates.code.stablebaselines, "get_environment.txt"),
        )

    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """



"""
Structure generator module.
"""

import argparse
import os
from typing import List, Tuple

from rpgt.src.generators.generator import Generator


class StructureGenerator(Generator):
    """
    Structure generator class.
    """

    def __init__(self, args: argparse.Namespace):
        """
        Initializes the structure generator.
        """
        super().__init__(args)
        self.files: List[Tuple[str, str]] = []
        self.folders: List[Tuple[str, bool]] = []

    def add_folder(self, path: str, is_module: bool = True) -> None:
        """
        Adds a folder at specified path.
        """
        self.folders.append((path, is_module))

    def add_file(self, path: str, content: str) -> None:
        """
        Adds a file at specified path.
        """
        content = content.replace("[=NAME=]", self.args.name)
        self.files.append((path, content))

    def add_src_file(self, path: str, content: str) -> None:
        """
        Adds a source file at specified path.
        """
        content = content.replace("[=NAME=]", self.args.name)
        self.add_file(f"src/{self.args.name}/{path}", content)

    def add_unit_test_file(self, path: str, content: str) -> None:
        """
        Adds a unit test file at specified path.
        """
        content = content.replace("[=NAME=]", self.args.name)
        self.add_file(f"tests/unit_tests/{self.args.name}/{path}", content)

    def generate(self, cwd: str) -> None:
        """
        Writes the structure to files.
        """
        for path, is_module in self.folders:
            os.mkdir(f"{path}")
            if is_module:
                with open(f"{cwd}/{path}/__init__.py", "w"):
                    pass

        for path, content in self.files:
            with open(f"{cwd}/{path}", "w") as file:
                file.write(content)



"""
PipEnv setup module.
"""

from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.environment.pipenv_generator import PipEnvGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.environment.environment_setup import EnvironmentSetup


class PipEnvSetup(EnvironmentSetup):
    """
    PipEnv setup class.
    """

    def get_standard_env(self) -> PipEnvGenerator:
        """
        Gets standard environment.
        :return: the standard environment.
        """
        return PipEnvGenerator(self.args)

    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """
        cicd_gen.add_job(
            "install",
            {
                "stage": "install",
                "script": ["pipenv install --dev"],
            },
        )

    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """

    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """



"""
The setup file for this package
"""
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rpgt",
    version="1.0.0",
    description="Automated Repository and Pipeline Generation for Data-Science Projects.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/tweet.dimitri/rpgt",
    license="MIT",
    packages=setuptools.find_packages(),
    include_package_data=True,
    python_requires=">=3.6",
    install_requires=[
        "pipenv",
        "mllint",
        "isort",
        "pylint",
        "mypy",
        "bandit",
        "black",
        "dvc",
    ],
    entry_points={
        "console_scripts": [
            "rpgt=rpgt.src.__main__:main",
        ]
    },
)



"""
 Package setup module.
 """

from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.setup import Setup


class PackageSetup(Setup):
    """
    Package setup class.
    """

    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """
        env_gen.add_requirement("setuptools", "~=57.0.0", dev_only=True)

    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """
        cicd_gen.add_job(
            "publish-package",
            {
                "stage": "production",
                "script": [
                    "pip install wheel twine",
                    "python3 setup.py sdist bdist_wheel",
                    "TWINE_PASSWORD=${CI_JOB_TOKEN} "
                    "TWINE_USERNAME=version_control-ci-token "
                    "python -m twine upload --repository-url "
                    "https://gitlab.com/api/v4/projects/${CI_PROJECT_ID}/packages/pypi dist/*",
                ],
                "only": ["tags"],
            },
        )

    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """



"""
File system module.
"""
import os
import shutil


def clear_cwd(cwd: str) -> None:
    """
    Clears current working directory.
    """

    for filename in os.listdir(cwd):
        file_path = os.path.join(cwd, filename)
        try:
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
        except IOError as exception:
            print("Failed to delete %s. Reason: %s" % (file_path, exception))



"""
Readme generator module.
"""
import argparse

from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.generator import Generator


def generate_main_information(project: str) -> str:
    """
    Generates the main project information.
    :return: a stringual representation of the main project information.
    """
    return f"""# {project}

"""


def generate_next_steps() -> str:
    """
    Generates the next steps information.
    :return: a stringual representation of the next steps information.
    """
    return """## Getting Started
To get started with your generated repository, please first check the next steps available on [the wiki](https://gitlab.com/tweet.dimitri/rpgt/-/wikis/Next%20steps) of RPGT.

"""


def generate_reproducibility() -> str:
    """
    Generates the reproducibility information.
    :return: a stringual representation of the reproducibility information.
    """
    return """## Reproducibility
Because this package uses [DVC](https://dvc.org/), reproducing results becomes an easy task.
To run all the required scripts in the right order, you only need to run `pipenv run dvc repro` (provided that you have already run `pipenv install`).
This will execute the pipeline defined in `dvc.yaml` and run the correct scripts in the right order.
Afterwards, you'll be able to see the results in the `results` directory.

"""


def generate_license(license_type: str) -> str:
    """
    Generates the license information.
    :return: a stringual representation of the license information.
    """
    return f"""## License
    
This project is {license_type.upper()} licensed. Please see the [LICENSE](LICENSE.md) file for more information.

"""


def generate_authors() -> str:
    """
    Generates the author information.
    :return: a stringual representation of the author information.
    """
    return """## Authors

- [Author 1]
- [Author 2]

Please see the [CONTRIBUTING](CONTRIBUTING.md) file for more information on how to contribute to this project.
"""


class ReadmeGenerator(Generator):
    """
    Readme generator class.
    """

    def __init__(self, args: argparse.Namespace, env_gen: EnvironmentGenerator):
        """
        Abstract initialize function definition.
        """
        super().__init__(args)
        self.env_gen = env_gen

    def generate(self, cwd: str) -> None:
        """
        Writes the README.md to file.
        """
        project = self.args.name
        license_type = self.args.license.lower()

        with open(f"{cwd}/README.md", "w") as file:
            file.write(generate_main_information(project))
            file.write(generate_next_steps())
            file.write(self.generate_requirements())
            file.write(generate_reproducibility())
            file.write(generate_license(license_type))
            file.write(generate_authors())

    def generate_requirements(self) -> str:
        """
        Generates the requirements information.
        :return: a stringual representation of the requirements information.
        """
        reqs = []

        for requirement, _ in self.env_gen.requirements.items():
            reqs.append(f"- {requirement}")

        reqs_str = "\n".join(reqs)

        return f"""## Requirements
In order to contribute to this package, you will need to have the following components installed on your system:
{reqs_str}

You can easily install them using `pipenv install`.

"""



"""
Setup generator module.
"""

import argparse

from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.generator import Generator


class SetupGenerator(Generator):
    """
    Setup generator class.
    """

    def __init__(self, args: argparse.Namespace, env_gen: EnvironmentGenerator):
        """
        Initializes the setup generator.
        """
        super().__init__(args)
        self.env_gen = env_gen

    def generate(self, cwd: str) -> None:
        """
        Writes the setup to file.
        """
        reqs = []

        for requirement, data in self.env_gen.requirements.items():
            if not data["dev_only"]:
                reqs.append(f'"{requirement}"')

        reqs_str = ", ".join(reqs)

        text = "\n".join(
            [
                '"""',
                "setup.py file for package management",
                '"""\n',
                "import setuptools\n",
                'with open("README.md", "r", encoding="utf-8") as fh:',
                "    long_description = fh.read()\n",
                "setuptools.setup(",
                f'    name="{self.args.name}",',
                '    version="0.0.1",',
                '    package_dir={"": "src"},',
                '    packages=setuptools.find_packages(where="src"),',
                '    python_requires=">=3.6",',
                f"    install_requires=[{reqs_str}],",
                ")\n",
            ]
        )

        with open(f"{cwd}/setup.py", "w") as file:
            file.write(text)



"""
Abstract environment setup module.
"""
import abc

from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.setup.setup import Setup


class EnvironmentSetup(Setup):
    """
    Abstract environment setup class.
    """

    @abc.abstractmethod
    def get_standard_env(self) -> EnvironmentGenerator:
        """
        Gets standard environment.
        :return: the standard environment.
        """
        raise NotImplementedError("Unimplemented")



"""
Contributing generator module.
"""
import argparse

from rpgt.src.generators.generator import Generator
from rpgt.src.generators.structure_generator import StructureGenerator


class ContributingGenerator(Generator):
    """
    Contributing generator class.
    """

    def __init__(self, args: argparse.Namespace, struct_gen: StructureGenerator):
        """
        Initialize Changelog generator with proper parameters.
        """

        super().__init__(args)
        self.struct_gen = struct_gen

    def generate(self, cwd: str) -> None:
        """
        Writes the CONTRIBUTING.md to file.
        """
        self.struct_gen.add_file("CONTRIBUTING.md", self.generate_contributing())

    @staticmethod
    def generate_contributing() -> str:
        """
        Generates the CONTRIBUTING.md file.
        :return: a stringual representation of the contributing file.
        """
        return """# Contributing Guide
We are excited that you are interested in contributing to this project. Before submitting your contribution, please make sure to take a moment and read through the following guidelines:

- [Code of conduct](#code-of-conduct)
- [Issue Reporting Guidelines](#issue-reporting-guidelines)
- [Pull Request Guidelines](#pull-request-guidelines)
- [Semantic Versioning](#semantic-versioning)
- [Style Guide](#style-guide)

## Code of Conduct
As contributors and maintainers of this project, we pledge to respect all people who contribute through reporting issues, posting feature requests, updating documentation, submitting pull requests or patches, and other activities.

We are committed to making participation in this project a harassment-free experience for everyone, regardless of the level of experience, gender, gender identity and expression, sexual orientation, disability, personal appearance, body size, race, age, or religion.

Examples of unacceptable behavior by participants include the use of sexual language or imagery, derogatory comments or personal attacks, trolling, public or private harassment, insults, or other unprofessional conduct.

Project maintainers have the right and responsibility to remove, edit, or reject comments, commits, code, wiki edits, issues, and other contributions that are not aligned to this Code of Conduct. Project maintainers who do not follow the Code of Conduct may be removed from the project team.

Instances of abusive, harassing, or otherwise unacceptable behavior may be reported by opening an issue or contacting one or more of the project maintainers.

This Code of Conduct is adapted from the [Contributor Covenant](http://contributor-covenant.org), version 1.0.0, available at [http://contributor-covenant.org/version/1/0/0/](http://contributor-covenant.org/version/1/0/0/).


## Issue Reporting Guidelines
Please use our `Bug.md` template to report bugs, and our `Feature.md` template to request a new feature.

## Pull Request Guidelines
Please see use our merge request template to open up a new pull or merge request.

## Semantic Versioning
This project follows semantic versioning. We release patch versions for critical bugfixes, minor versions for new features or non-essential changes, and major versions for any breaking changes.

Every significant change is documented in the `CHANGELOG.md` file.

To release a new version, please mind the following:
- Change the version number in the `setup.py` file
- Use `git push --tags`

## Branch Organization
Submit all changes directly to the master branch. We don't use separate branches for development or for upcoming releases. We do our best to keep master in good shape, with all tests passing.

Code that lands in master must be compatible with the latest stable release. It may contain additional features, but no breaking changes. We should be able to release a new minor version from the tip of master at any time.

## Style Guide
This project uses [mllint](https://github.com/bvobart/mllint) to evaluate its technical quality. You can check the status of your code styling by simply running `mllint`.
"""



"""
Field setup module.
"""
from rpgt.src.generators.cicd.cicd_generator import CICDGenerator
from rpgt.src.generators.environment.environment_generator import EnvironmentGenerator
from rpgt.src.generators.structure_generator import StructureGenerator
from rpgt.src.setup.setup import Setup


class FieldSetup(Setup):
    """
    Field setup class.
    """

    def setup_dependencies(self, env_gen: EnvironmentGenerator) -> None:
        """
        Sets up project dependencies.
        """

    def setup_files(self, struct_gen: StructureGenerator) -> None:
        """
        Sets up project files.
        """

    def setup_pipeline(self, cicd_gen: CICDGenerator) -> None:
        """
        Sets up project pipeline.
        """



"""
Field factory test file
"""
import argparse

from rpgt.src.factories.field_factory import get_field


def test_factory() -> None:
    """
    Tests the ML setup
    :return: whether the assertion failed or not
    """
    parser = argparse.ArgumentParser(
        description="Generator to automatically set up your new project!"
    )
    args, _ = parser.parse_known_args()

    get_field("RL", args, "")
